<?php

namespace Drupal\ai_upgrade_assistant\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\State\StateInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Service for interacting with OpenAI API.
 */
class OpenAIService {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Maximum number of retries for API calls.
   */
  const MAX_RETRIES = 3;

  /**
   * Base delay between retries in seconds.
   */
  const BASE_RETRY_DELAY = 1;

  /**
   * Base URL for the OpenAI API.
   *
   * @var string
   */
  protected const API_BASE_URL = 'https://api.openai.com/v1/';

  /**
   * Constructs a new OpenAIService.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   */
  public function __construct(
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    StateInterface $state,
    CacheBackendInterface $cache
  ) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->state = $state;
    $this->cache = $cache;
  }

  /**
   * Gets the configured OpenAI API key.
   *
   * @return string
   *   The configured API key.
   *
   * @throws \Exception
   *   If the API key is not configured or invalid.
   */
  protected function getApiKey() {
    $api_key = $this->state->get('ai_upgrade_assistant.openai_api_key');
    $logger = $this->loggerFactory->get('ai_upgrade_assistant');
    
    if (empty($api_key)) {
      $logger->error('OpenAI API key not configured');
      throw new \Exception(
        'OpenAI API key is not configured. Please visit /admin/config/development/ai-upgrade-assistant to configure your API key.'
      );
    }

    // Temporary debug logging - DO NOT COMMIT
    $logger->debug('Full API key for debugging: @key', ['@key' => $api_key]);
    
    // Add debug logging
    $logger->debug('Using OpenAI API key: @key', [
      '@key' => substr($api_key, 0, 8) . str_repeat('*', strlen($api_key) - 12) . substr($api_key, -4)
    ]);

    return $api_key;
  }

  /**
   * Makes a request to the OpenAI API with retries.
   *
   * @param string $endpoint
   *   The API endpoint.
   * @param array $data
   *   The request data.
   * @param int $attempt
   *   Current attempt number.
   *
   * @return array
   *   The API response data.
   *
   * @throws \Exception
   *   If all retries fail.
   */
  protected function makeApiRequest($endpoint, array $data, $attempt = 1) {
    try {
      $api_key = $this->getApiKey();
      $logger = $this->loggerFactory->get('ai_upgrade_assistant');
      
      // Log request details
      $logger->debug('Making request to @endpoint with API key: @key', [
        '@endpoint' => $endpoint,
        '@key' => substr($api_key, 0, 12) . '...' . substr($api_key, -4),
      ]);

      // Check rate limits
      $limits = $this->getRateLimitStatus();
      if (!empty($limits['requests_remaining']) && $limits['requests_remaining'] < 1) {
        $wait_time = max(0, $limits['reset_time'] - time());
        if ($wait_time > 0) {
          $logger->warning(
            'Rate limit reached. Waiting @seconds seconds.',
            ['@seconds' => $wait_time]
          );
          sleep($wait_time);
        }
      }

      // Log full request details for debugging
      $logger->debug('Full request details: @details', [
        '@details' => json_encode([
          'endpoint' => $endpoint,
          'headers' => [
            'Authorization' => 'Bearer ' . substr($api_key, 0, 12) . '...' . substr($api_key, -4),
            'Content-Type' => 'application/json',
          ],
          'data' => $data,
        ], JSON_PRETTY_PRINT),
      ]);
      
      $response = $this->httpClient->post($endpoint, [
        'headers' => [
          'Authorization' => 'Bearer ' . $api_key,
          'Content-Type' => 'application/json',
        ],
        'json' => $data,
        'http_errors' => false,
      ]);

      $status_code = $response->getStatusCode();
      $body = (string) $response->getBody();
      $result = json_decode($body, TRUE);

      // Log response details
      $logger->debug('Response details: @details', [
        '@details' => json_encode([
          'status' => $status_code,
          'headers' => $response->getHeaders(),
          'body' => $result,
        ], JSON_PRETTY_PRINT),
      ]);

      // Update rate limits from response headers
      $this->updateRateLimitStatus($response->getHeaders());

      // Handle different status codes
      switch ($status_code) {
        case 200:
          return $result;

        case 401:
          $error_msg = isset($result['error']) ? json_encode($result['error']) : 'Invalid OpenAI API key';
          $logger->error('Authentication failed: @error', ['@error' => $error_msg]);
          throw new \Exception('OpenAI API authentication failed: ' . $error_msg);

        case 429: // Rate limit exceeded
          if ($attempt < self::MAX_RETRIES) {
            $retry_after = $response->getHeader('Retry-After')[0] ?? self::BASE_RETRY_DELAY * $attempt;
            $logger->warning('Rate limit exceeded. Retrying in @seconds seconds.', ['@seconds' => $retry_after]);
            sleep($retry_after);
            return $this->makeApiRequest($endpoint, $data, $attempt + 1);
          }
          throw new \Exception('Rate limit exceeded and max retries reached.');

        case 500:
        case 502:
        case 503:
        case 504:
          if ($attempt < self::MAX_RETRIES) {
            $delay = self::BASE_RETRY_DELAY * $attempt;
            $logger->warning('Server error (@code). Retrying in @seconds seconds.', 
              ['@code' => $status_code, '@seconds' => $delay]
            );
            sleep($delay);
            return $this->makeApiRequest($endpoint, $data, $attempt + 1);
          }
          throw new \Exception("Server error: $status_code");

        default:
          $error_message = isset($result['error']) ? json_encode($result['error']) : "Unknown error (HTTP $status_code)";
          throw new \Exception($error_message);
      }
    }
    catch (GuzzleException $e) {
      if ($attempt < self::MAX_RETRIES) {
        $delay = self::BASE_RETRY_DELAY * $attempt;
        $logger->warning('Request failed: @error. Retrying in @seconds seconds.',
          ['@error' => $e->getMessage(), '@seconds' => $delay]
        );
        sleep($delay);
        return $this->makeApiRequest($endpoint, $data, $attempt + 1);
      }
      throw new \Exception('Failed to connect to OpenAI API: ' . $e->getMessage());
    }
  }

  /**
   * Generates a completion using OpenAI API.
   *
   * @param string $prompt
   *   The prompt to generate completion for.
   *
   * @return string
   *   The generated completion.
   *
   * @throws \Exception
   *   If the API request fails.
   */
  public function generateCompletion($prompt) {
    $cid = 'openai_completion:' . md5($prompt);
    if ($cached = $this->cache->get($cid)) {
      return $cached->data;
    }

    try {
      $result = $this->makeApiRequest(self::API_BASE_URL . 'chat/completions', [
        'model' => 'gpt-4',
        'messages' => [
          [
            'role' => 'user',
            'content' => $prompt,
          ],
        ],
        'temperature' => 0.7,
        'max_tokens' => 1000,
      ]);

      if (empty($result['choices'][0]['message']['content'])) {
        throw new \Exception('Invalid response from OpenAI API.');
      }

      $completion = $result['choices'][0]['message']['content'];
      $this->cache->set($cid, $completion, time() + 3600);

      return $completion;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('ai_upgrade_assistant')->error(
        'OpenAI API request failed: @error',
        ['@error' => $e->getMessage()]
      );
      throw $e;
    }
  }

  /**
   * Analyzes code for potential upgrade issues.
   *
   * @param array|string $code
   *   The code to analyze.
   * @param array $context
   *   Additional context for the analysis.
   *
   * @return array
   *   Analysis results.
   *
   * @throws \Exception
   *   If the API request fails.
   */
  public function analyzeCode($code, array $context = []) {
    // If code is an array (AST), convert it to a string representation
    if (is_array($code)) {
      $code = var_export($code, true);
    }

    $cid = 'openai_code_analysis:' . md5($code . serialize($context));
    if ($cached = $this->cache->get($cid)) {
      return $cached->data;
    }

    try {
      $prompt = $this->buildCodeAnalysisPrompt($code, $context);
      
      $result = $this->makeApiRequest(self::API_BASE_URL . 'chat/completions', [
        'model' => 'gpt-4',
        'messages' => [
          [
            'role' => 'system',
            'content' => 'You are a Drupal code analyzer specializing in identifying compatibility issues and suggesting fixes for module upgrades.',
          ],
          [
            'role' => 'user',
            'content' => $prompt,
          ],
        ],
        'temperature' => 0.3,
        'max_tokens' => 1000,
      ]);
      
      if (empty($result['choices'][0]['message']['content'])) {
        throw new \Exception('Invalid response from OpenAI API.');
      }

      $analysis = $this->parseAnalysisResponse($result['choices'][0]['message']['content']);
      
      // Cache for 24 hours
      $this->cache->set($cid, $analysis, time() + 86400);
      
      return $analysis;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('ai_upgrade_assistant')->error(
        'OpenAI API request failed: @error',
        ['@error' => $e->getMessage()]
      );
      throw $e;
    }
  }

  /**
   * Analyzes code for security issues using OpenAI.
   *
   * @param string $prompt
   *   The analysis prompt containing code context.
   *
   * @return array
   *   Analysis results containing identified security issues.
   *
   * @throws \Exception
   *   If the API request fails.
   */
  public function analyzeSecurity($prompt) {
    $system_message = "You are a security expert analyzing Drupal code for potential security vulnerabilities. Focus on:
- SQL injection
- XSS vulnerabilities
- CSRF issues
- Access bypass vulnerabilities
- File system security
- Input validation
- Sanitization issues
Provide specific, actionable feedback for each issue found.";

    $messages = [
      ['role' => 'system', 'content' => $system_message],
      ['role' => 'user', 'content' => $prompt],
    ];

    try {
      $response = $this->chatCompletion($messages);
      
      if (!empty($response['choices'][0]['message']['content'])) {
        $analysis = json_decode($response['choices'][0]['message']['content'], TRUE);
        
        if (json_last_error() === JSON_ERROR_NONE && isset($analysis['issues'])) {
          return $analysis;
        }
        
        // If response isn't valid JSON but we got content, try to parse it
        return [
          'issues' => [
            [
              'type' => 'general',
              'description' => $response['choices'][0]['message']['content'],
              'severity' => 'unknown',
            ],
          ],
        ];
      }
      
      return ['issues' => []];
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('ai_upgrade_assistant')
        ->error('Security analysis failed: @error', ['@error' => $e->getMessage()]);
      throw $e;
    }
  }

  /**
   * Builds the prompt for code analysis.
   *
   * @param string $code
   *   The code to analyze.
   * @param array $context
   *   Additional context for analysis.
   *
   * @return string
   *   The formatted prompt.
   */
  protected function buildCodeAnalysisPrompt($code, array $context) {
    $prompt = "Analyze the following Drupal code for compatibility issues:\n\n";
    $prompt .= "Code:\n```php\n$code\n```\n\n";
    
    if (!empty($context['from_version'])) {
      $prompt .= "Current Drupal version: {$context['from_version']}\n";
    }
    if (!empty($context['to_version'])) {
      $prompt .= "Target Drupal version: {$context['to_version']}\n";
    }
    
    $prompt .= "\nPlease identify:\n";
    $prompt .= "1. Deprecated code usage\n";
    $prompt .= "2. Incompatible API changes\n";
    $prompt .= "3. Potential security issues\n";
    $prompt .= "4. Suggested fixes with code examples\n";
    
    return $prompt;
  }

  /**
   * Parses the OpenAI response into structured analysis data.
   *
   * @param string $response
   *   The raw response from OpenAI.
   *
   * @return array
   *   Structured analysis data.
   */
  protected function parseAnalysisResponse($response) {
    // Basic structure for the analysis
    $analysis = [
      'issues' => [],
      'suggestions' => [],
      'compatibility' => 'unknown',
    ];

    // Extract issues (assumed to be listed with numbers or bullet points)
    if (preg_match_all('/(?:^|\n)(?:\d+\.|-)(.+?)(?=(?:\n\d+\.|\n-|\n\n|$))/s', $response, $matches)) {
      foreach ($matches[1] as $issue) {
        $analysis['issues'][] = trim($issue);
      }
    }

    // Extract code suggestions (anything between ```php and ```)
    if (preg_match_all('/```php\n(.+?)\n```/s', $response, $matches)) {
      foreach ($matches[1] as $suggestion) {
        $analysis['suggestions'][] = trim($suggestion);
      }
    }

    // Determine overall compatibility
    if (stripos($response, 'compatible') !== false) {
      if (stripos($response, 'not compatible') !== false || stripos($response, 'incompatible') !== false) {
        $analysis['compatibility'] = 'incompatible';
      }
      else {
        $analysis['compatibility'] = 'compatible';
      }
    }

    return $analysis;
  }

  /**
   * Gets the current rate limit status.
   *
   * @return array
   *   The rate limit status.
   */
  public function getRateLimitStatus() {
    return $this->state->get('ai_upgrade_assistant.openai_rate_limit', [
      'requests_remaining' => 0,
      'requests_reset_at' => 0,
    ]);
  }

  /**
   * Updates the rate limit status.
   *
   * @param array $headers
   *   The response headers from OpenAI API.
   */
  protected function updateRateLimitStatus($headers) {
    $status = [
      'requests_remaining' => $headers['x-ratelimit-remaining-requests'][0] ?? 0,
      'requests_reset_at' => $headers['x-ratelimit-reset-requests'][0] ?? 0,
    ];
    $this->state->set('ai_upgrade_assistant.openai_rate_limit', $status);
  }

  /**
   * Get current rate limits for OpenAI API.
   *
   * @return array
   *   Array containing rate limit information:
   *   - requests_remaining: Number of requests remaining in the current window
   *   - tokens_remaining: Number of tokens remaining in the current window
   *   - reset_time: Timestamp when the rate limits will reset
   */
  public function getRateLimits() {
    $state = \Drupal::state();
    
    return [
      'requests_remaining' => $state->get('ai_upgrade_assistant.openai_requests_remaining', 100),
      'tokens_remaining' => $state->get('ai_upgrade_assistant.openai_tokens_remaining', 100000),
      'reset_time' => $state->get('ai_upgrade_assistant.openai_reset_time', strtotime('+1 hour')),
    ];
  }

  /**
   * Makes a request to the OpenAI API for chat completion.
   *
   * @param array $messages
   *   The messages to send to the model.
   *
   * @return array
   *   The API response data.
   *
   * @throws \Exception
   *   If the API request fails.
   */
  protected function chatCompletion(array $messages) {
    try {
      $api_key = $this->getApiKey();
      $logger = $this->loggerFactory->get('ai_upgrade_assistant');
      
      // Log request details
      $logger->debug('Making request to @endpoint with API key: @key', [
        '@endpoint' => self::API_BASE_URL . 'chat/completions',
        '@key' => substr($api_key, 0, 12) . '...' . substr($api_key, -4),
      ]);

      // Check rate limits
      $limits = $this->getRateLimitStatus();
      if (!empty($limits['requests_remaining']) && $limits['requests_remaining'] < 1) {
        $wait_time = max(0, $limits['reset_time'] - time());
        if ($wait_time > 0) {
          $logger->warning(
            'Rate limit reached. Waiting @seconds seconds.',
            ['@seconds' => $wait_time]
          );
          sleep($wait_time);
        }
      }

      // Log full request details for debugging
      $logger->debug('Full request details: @details', [
        '@details' => json_encode([
          'endpoint' => self::API_BASE_URL . 'chat/completions',
          'headers' => [
            'Authorization' => 'Bearer ' . substr($api_key, 0, 12) . '...' . substr($api_key, -4),
            'Content-Type' => 'application/json',
          ],
          'data' => $messages,
        ], JSON_PRETTY_PRINT),
      ]);
      
      $response = $this->httpClient->post(self::API_BASE_URL . 'chat/completions', [
        'headers' => [
          'Authorization' => 'Bearer ' . $api_key,
          'Content-Type' => 'application/json',
        ],
        'json' => $messages,
        'http_errors' => false,
      ]);

      $status_code = $response->getStatusCode();
      $body = (string) $response->getBody();
      $result = json_decode($body, TRUE);

      // Log response details
      $logger->debug('Response details: @details', [
        '@details' => json_encode([
          'status' => $status_code,
          'headers' => $response->getHeaders(),
          'body' => $result,
        ], JSON_PRETTY_PRINT),
      ]);

      // Update rate limits from response headers
      $this->updateRateLimitStatus($response->getHeaders());

      // Handle different status codes
      switch ($status_code) {
        case 200:
          return $result;

        case 401:
          $error_msg = isset($result['error']) ? json_encode($result['error']) : 'Invalid OpenAI API key';
          $logger->error('Authentication failed: @error', ['@error' => $error_msg]);
          throw new \Exception('OpenAI API authentication failed: ' . $error_msg);

        case 429: // Rate limit exceeded
          if ($attempt < self::MAX_RETRIES) {
            $retry_after = $response->getHeader('Retry-After')[0] ?? self::BASE_RETRY_DELAY * $attempt;
            $logger->warning('Rate limit exceeded. Retrying in @seconds seconds.', ['@seconds' => $retry_after]);
            sleep($retry_after);
            return $this->makeApiRequest(self::API_BASE_URL . 'chat/completions', $messages, $attempt + 1);
          }
          throw new \Exception('Rate limit exceeded and max retries reached.');

        case 500:
        case 502:
        case 503:
        case 504:
          if ($attempt < self::MAX_RETRIES) {
            $delay = self::BASE_RETRY_DELAY * $attempt;
            $logger->warning('Server error (@code). Retrying in @seconds seconds.', 
              ['@code' => $status_code, '@seconds' => $delay]
            );
            sleep($delay);
            return $this->makeApiRequest(self::API_BASE_URL . 'chat/completions', $messages, $attempt + 1);
          }
          throw new \Exception("Server error: $status_code");

        default:
          $error_message = isset($result['error']) ? json_encode($result['error']) : "Unknown error (HTTP $status_code)";
          throw new \Exception($error_message);
      }
    }
    catch (GuzzleException $e) {
      if ($attempt < self::MAX_RETRIES) {
        $delay = self::BASE_RETRY_DELAY * $attempt;
        $logger->warning('Request failed: @error. Retrying in @seconds seconds.',
          ['@error' => $e->getMessage(), '@seconds' => $delay]
        );
        sleep($delay);
        return $this->makeApiRequest(self::API_BASE_URL . 'chat/completions', $messages, $attempt + 1);
      }
      throw new \Exception('Failed to connect to OpenAI API: ' . $e->getMessage());
    }
  }

}
