<?php

namespace Drupal\ai_upgrade_assistant\Service;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use PhpParser\Node;
use PhpParser\Node\Expr\FuncCall;
use PhpParser\Node\Name;
use PhpParser\NodeFinder;
use PhpParser\NodeTraverser;
use PhpParser\NodeVisitorAbstract;
use PhpParser\NodeVisitor\NameResolver;
use GuzzleHttp\ClientInterface;
use Drupal\ai_upgrade_assistant\Service\OpenAIService;
use Drupal\ai_upgrade_assistant\Service\PhpParserService;
use Drupal\ai_upgrade_assistant\Service\AiModelManager;
use Drupal\ai_upgrade_assistant\Service\CommunityLearningService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Service for analyzing Drupal projects for upgrades using AI.
 */
class ProjectAnalyzer implements ContainerInjectionInterface {
  use DependencySerializationTrait;
  use StringTranslationTrait;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The OpenAI service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\OpenAIService
   */
  protected $openAi;

  /**
   * The PHP parser service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\PhpParserService
   */
  protected $phpParser;

  /**
   * The AI model manager.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\AiModelManager
   */
  protected $aiModelManager;

  /**
   * The community learning service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\CommunityLearningService
   */
  protected $communityLearning;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs a new ProjectAnalyzer.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $moduleExtensionList
   *   The module extension list service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory service.
   * @param \Drupal\ai_upgrade_assistant\Service\OpenAIService $openAi
   *   The OpenAI service.
   * @param \Drupal\ai_upgrade_assistant\Service\PhpParserService $phpParser
   *   The PHP parser service.
   * @param \Drupal\ai_upgrade_assistant\Service\AiModelManager $aiModelManager
   *   The AI model manager service.
   * @param \Drupal\ai_upgrade_assistant\Service\CommunityLearningService $communityLearning
   *   The community learning service.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   The HTTP client.
   */
  public function __construct(
    ModuleHandlerInterface $moduleHandler,
    ModuleExtensionList $moduleExtensionList,
    ConfigFactoryInterface $configFactory,
    FileSystemInterface $fileSystem,
    LoggerChannelFactoryInterface $loggerFactory,
    OpenAIService $openAi,
    PhpParserService $phpParser,
    AiModelManager $aiModelManager,
    CommunityLearningService $communityLearning,
    ClientInterface $httpClient
  ) {
    $this->moduleHandler = $moduleHandler;
    $this->moduleExtensionList = $moduleExtensionList;
    $this->configFactory = $configFactory;
    $this->fileSystem = $fileSystem;
    $this->logger = $loggerFactory->get('ai_upgrade_assistant');
    $this->openAi = $openAi;
    $this->phpParser = $phpParser;
    $this->aiModelManager = $aiModelManager;
    $this->communityLearning = $communityLearning;
    $this->httpClient = $httpClient;
  }

  /**
   * Creates a new instance of the service.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   *
   * @return static
   *   The created instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('extension.list.module'),
      $container->get('config.factory'),
      $container->get('file_system'),
      $container->get('logger.factory'),
      $container->get('ai_upgrade_assistant.openai'),
      $container->get('ai_upgrade_assistant.php_parser'),
      $container->get('ai_upgrade_assistant.ai_model_manager'),
      $container->get('ai_upgrade_assistant.community_learning'),
      $container->get('http_client')
    );
  }

  /**
   * Ensures all required services are loaded after deserialization.
   */
  protected function ensureServices() {
    if (!$this->moduleHandler) {
      $this->moduleHandler = \Drupal::service('module_handler');
    }
    if (!$this->moduleExtensionList) {
      $this->moduleExtensionList = \Drupal::service('extension.list.module');
    }
    if (!$this->configFactory) {
      $this->configFactory = \Drupal::configFactory();
    }
    if (!$this->fileSystem) {
      $this->fileSystem = \Drupal::service('file_system');
    }
    if (!$this->logger) {
      $this->logger = \Drupal::logger('ai_upgrade_assistant');
    }
    if (!$this->openAi) {
      $this->openAi = \Drupal::service('ai_upgrade_assistant.openai');
    }
    if (!$this->phpParser) {
      $this->phpParser = \Drupal::service('ai_upgrade_assistant.php_parser');
    }
    if (!$this->aiModelManager) {
      $this->aiModelManager = \Drupal::service('ai_upgrade_assistant.ai_model_manager');
    }
    if (!$this->communityLearning) {
      $this->communityLearning = \Drupal::service('ai_upgrade_assistant.community_learning');
    }
    if (!$this->httpClient) {
      $this->httpClient = \Drupal::httpClient();
    }
  }

  /**
   * Analyzes code for potential upgrade issues.
   *
   * @param string $path
   *   The path to analyze.
   *
   * @return array
   *   Analysis results.
   */
  public function analyzeCode($path) {
    // Ensure services are properly loaded after serialization
    $this->ensureServices();
    
    $results = [];
    $files = is_dir($path) ? $this->findPhpFiles($path) : [$path];

    foreach ($files as $file) {
      try {
        // Parse the file into an AST
        $ast = $this->phpParser->parseFile($file);
        if (!$ast) {
          $this->logger->warning('Could not parse file @file', ['@file' => $file]);
          continue;
        }

        // Build analysis context
        $file_config = $this->buildCodeContext($ast, $file);

        // Get similar patterns from community
        $patterns = $this->communityLearning->findSimilarPatterns($file);
        if ($patterns) {
          $file_config['patterns'] = array_map(function ($pattern) {
            return is_object($pattern) && isset($pattern->pattern_data) 
              ? unserialize($pattern->pattern_data) 
              : [];
          }, $patterns);
        }

        // Convert AST to string for OpenAI analysis
        $code_string = file_get_contents($file);
        $results[$file] = $this->openAi->analyzeCode($code_string, $file_config);
      }
      catch (\Exception $e) {
        $this->logger->error('Error analyzing file @file: @error', [
          '@file' => $file,
          '@error' => $e->getMessage(),
        ]);
        $results[$file] = [
          'status' => 'error',
          'message' => $e->getMessage(),
        ];
      }
    }

    return $results;
  }

  /**
   * Builds context for code analysis.
   *
   * @param array $ast
   *   The AST of the code.
   * @param string $file
   *   The file being analyzed.
   *
   * @return array
   *   Code context for analysis.
   */
  protected function buildCodeContext(array $ast, $file) {
    return [
      'file_path' => $file,
      'ast' => $ast,
      'drupal_version' => \Drupal::VERSION,
      'module_info' => $this->getModuleInfo($file),
      'dependencies' => $this->getDependencies($file),
    ];
  }

  /**
   * Gets module info for a file.
   *
   * @param string $file
   *   File path.
   *
   * @return array
   *   Module information.
   */
  protected function getModuleInfo($file) {
    $module_name = $this->getModuleNameFromPath($file);
    $module_info = [];

    try {
      if ($module_name) {
        $module_info = $this->moduleExtensionList->getExtensionInfo($module_name);
      }
    }
    catch (\Exception $e) {
      $this->logger->warning('Could not get module info for @module: @error', [
        '@module' => $module_name,
        '@error' => $e->getMessage(),
      ]);
    }

    return $module_info;
  }

  /**
   * Gets dependencies for a file.
   *
   * @param string $file
   *   File path.
   *
   * @return array
   *   Dependencies information.
   */
  public function getDependencies($file) {
    $dependencies = [];
    $module_info = $this->getModuleInfo($file);

    // Add core dependencies based on namespace and class
    if (strpos($file, '/Plugin/Field/') !== FALSE) {
      $dependencies['field'] = 'field';
    }
    elseif (strpos($file, '/Plugin/Block/') !== FALSE) {
      $dependencies['block'] = 'block';
    }
    elseif (strpos($file, '/Plugin/views/') !== FALSE) {
      $dependencies['views'] = 'views';
    }

    // Add dependencies from module info
    if (!empty($module_info['dependencies'])) {
      foreach ($module_info['dependencies'] as $dependency) {
        // Parse dependency string (e.g., "drupal:views (>=8.x-3.x)")
        if (preg_match('/^([a-z0-9_]+):([a-z0-9_]+)/', $dependency, $matches)) {
          $dependencies[$matches[2]] = $matches[2];
        }
        else {
          $dependencies[$dependency] = $dependency;
        }
      }
    }

    return $dependencies;
  }

  /**
   * Finds all PHP files in a project.
   *
   * @param string $project_path
   *   Path to the project.
   *
   * @return array
   *   List of PHP files.
   */
  protected function findPhpFiles($project_path) {
    $files = [];
    $iterator = new \RecursiveIteratorIterator(
      new \RecursiveDirectoryIterator($project_path)
    );

    foreach ($iterator as $file) {
      if ($file->isFile() && $file->getExtension() === 'php') {
        $files[] = $file->getPathname();
      }
    }

    return $files;
  }

  /**
   * Gets the module name from a file path.
   *
   * @param string $file_path
   *   The file path.
   *
   * @return string|null
   *   The module name, or null if it cannot be determined.
   */
  protected function getModuleNameFromPath($file_path) {
    // First check if file is in a module's directory
    if (preg_match('#modules/(?:contrib|custom)/([^/]+)(?:/[^/]+)?/#', $file_path, $matches)) {
      return $matches[1];
    }

    // For core modules, check core/modules path
    if (preg_match('#core/modules/([^/]+)/#', $file_path, $matches)) {
      return $matches[1];
    }

    // For profile modules
    if (preg_match('#profiles/(?:contrib|custom)/([^/]+)/#', $file_path, $matches)) {
      return $matches[1];
    }

    // If we can't determine module from path, try to get it from namespace
    try {
      $ast = $this->phpParser->parseFile($file_path);
      if ($ast) {
        $namespace = '';
        $visitor = new class($namespace) extends NodeVisitorAbstract {
          private $namespace;
          public function __construct(&$namespace) {
            $this->namespace = &$namespace;
          }
          public function enterNode(Node $node) {
            if ($node instanceof Node\Stmt\Namespace_) {
              $this->namespace = implode('\\', $node->name->parts);
            }
          }
        };
        
        $traverser = new NodeTraverser();
        $traverser->addVisitor($visitor);
        $traverser->traverse($ast);

        if ($namespace && preg_match('/^Drupal\\\\([^\\\\]+)/', $namespace, $matches)) {
          return strtolower($matches[1]);
        }
      }
    }
    catch (\Exception $e) {
      $this->logger->debug('Could not parse file for namespace: @error', [
        '@error' => $e->getMessage(),
      ]);
    }

    return NULL;
  }

  /**
   * Analyzes security issues for a module.
   *
   * @param string $module
   *   The module name.
   *
   * @return array
   *   Security analysis results.
   */
  public function analyzeSecurityIssues($module) {
    try {
      // Try to get module info from both extension list and module handler
      $module_data = null;
      try {
        $module_data = $this->moduleExtensionList->get($module);
      }
      catch (\Exception $e) {
        try {
          $module_data = $this->moduleHandler->getModule($module);
        }
        catch (\Exception $e) {
          return [
            'status' => 'error',
            'message' => $this->t('Module @module not found', ['@module' => $module]),
          ];
        }
      }

      if (!$module_data) {
        return [
          'status' => 'error',
          'message' => $this->t('Module @module not found', ['@module' => $module]),
        ];
      }

      // Get known security advisories
      try {
        $advisories = $this->fetchSecurityAdvisories($module);
      }
      catch (\Exception $e) {
        $this->logger->error('Error fetching security advisories for @module: @error', [
          '@module' => $module,
          '@error' => $e->getMessage(),
        ]);
        $advisories = [];
      }
      
      // Analyze code for potential security issues
      $path = $module_data->getPath();
      if (!$this->fileSystem->realpath($path)) {
        return [
          'status' => 'error',
          'message' => $this->t('Module path @path not found', ['@path' => $path]),
        ];
      }

      $files = $this->findPhpFiles($path);
      $code_issues = [];
      
      foreach ($files as $file) {
        try {
          $ast = $this->phpParser->parseFile($file);
          if (!$ast) {
            $this->logger->warning('Unable to parse file @file', ['@file' => $file]);
            continue;
          }
          
          $file_issues = $this->analyzeCodeSecurity($ast, $file);
          
          if (!empty($file_issues)) {
            $code_issues[$file] = $file_issues;
          }
        }
        catch (\Exception $e) {
          $this->logger->error('Error analyzing file @file: @error', [
            '@file' => $file,
            '@error' => $e->getMessage(),
          ]);
        }
      }

      // Calculate overall risk level
      $risk_level = $this->calculateSecurityRiskLevel($advisories, $code_issues);

      return [
        'status' => 'analyzed',
        'risk_level' => $risk_level,
        'advisories' => $advisories,
        'code_issues' => $code_issues,
      ];
    }
    catch (\Exception $e) {
      $this->logger->error('Error analyzing security for module @module: @error', [
        '@module' => $module,
        '@error' => $e->getMessage(),
      ]);
      return [
        'status' => 'error',
        'message' => $this->t('Error analyzing module security: @error', ['@error' => $e->getMessage()]),
      ];
    }
  }

  /**
   * Fetches security advisories for a given module.
   *
   * @param string $module_name
   *   The name of the module.
   *
   * @return array
   *   Array of security advisories.
   */
  protected function fetchSecurityAdvisories($module_name) {
    try {
      // Use the Drupal.org API endpoint for security advisories
      $url = "https://www.drupal.org/api-d7/node.json?type=sa&field_project=" . urlencode($module_name);
      
      $this->logger->debug('Fetching security advisories from @url', ['@url' => $url]);
      
      $response = $this->httpClient->get($url, [
        'headers' => [
          'Accept' => 'application/json',
        ],
        'verify' => TRUE,
      ]);
      
      if ($response->getStatusCode() == 200) {
        $data = json_decode($response->getBody(), TRUE);
        
        if (isset($data['list'])) {
          $advisories = [];
          foreach ($data['list'] as $advisory) {
            $advisories[] = [
              'title' => $advisory['title'] ?? 'Unknown Advisory',
              'link' => isset($advisory['nid']) ? "https://www.drupal.org/node/{$advisory['nid']}" : '',
              'severity' => $advisory['field_sa_risk_level'] ?? 'Unknown',
              'date' => isset($advisory['created']) ? date('Y-m-d', $advisory['created']) : '',
              'project' => $module_name,
              'version' => $advisory['field_sa_version'] ?? '',
            ];
          }
          return $advisories;
        }
      }
      
      // If we get here, either the response wasn't 200 or the data wasn't in the expected format
      $this->logger->warning('No security advisories found for @module (Status: @status)', [
        '@module' => $module_name,
        '@status' => $response->getStatusCode(),
      ]);
      
      return [];
    }
    catch (\Exception $e) {
      $this->logger->error('Error fetching security advisories for @module: @error', [
        '@module' => $module_name,
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Gets available updates for a module.
   *
   * @param string $module_name
   *   The name of the module.
   *
   * @return array
   *   Array of available updates.
   */
  protected function getAvailableUpdates($module_name) {
    try {
      // Use the releases API endpoint
      $url = "https://updates.drupal.org/release-history/$module_name/current";
      
      $this->logger->debug('Fetching available updates from @url', ['@url' => $url]);
      
      $response = $this->httpClient->get($url);
      
      if ($response->getStatusCode() == 200) {
        $xml = simplexml_load_string($response->getBody());
        if ($xml === FALSE) {
          throw new \Exception('Failed to parse XML response');
        }
        
        $releases = [];
        if (isset($xml->releases->release)) {
          foreach ($xml->releases->release as $release) {
            $releases[] = [
              'version' => (string)$release->version,
              'date' => isset($release->date) ? date('Y-m-d', (int)$release->date) : '',
              'status' => (string)$release->status,
              'download_link' => (string)$release->download_link,
              'release_link' => (string)$release->release_link,
            ];
          }
        }
        
        return $releases;
      }
      
      $this->logger->warning('No updates found for @module (Status: @status)', [
        '@module' => $module_name,
        '@status' => $response->getStatusCode(),
      ]);
      
      return [];
    }
    catch (\Exception $e) {
      $this->logger->error('Error fetching updates for @module: @error', [
        '@module' => $module_name,
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Analyzes code for security issues.
   *
   * @param array $ast
   *   The AST of the code to analyze.
   * @param string $file
   *   The file being analyzed.
   *
   * @return array
   *   Array of security issues found.
   */
  protected function analyzeCodeSecurity($ast, $file) {
    $issues = [];
    
    try {
      // Get context about the code
      $context = $this->buildCodeContext($ast, $file);
      
      // Prepare the analysis prompt
      $prompt = $this->aiModelManager->prepareSecurityAnalysisPrompt($context);
      
      // Use OpenAI to analyze the code
      $analysis = $this->openAi->analyzeSecurity($prompt);
      
      if (!empty($analysis['issues'])) {
        $issues = $analysis['issues'];
      }
      
      // Log any identified issues
      if (!empty($issues)) {
        $this->logger->notice('Found @count security issues in @file', [
          '@count' => count($issues),
          '@file' => $file,
        ]);
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Error during security analysis of @file: @error', [
        '@file' => $file,
        '@error' => $e->getMessage(),
      ]);
    }
    
    return $issues;
  }

  /**
   * Calculates the overall security risk level based on findings.
   *
   * @param array $advisories
   *   Security advisories from drupal.org.
   * @param array $code_issues
   *   Security issues found in code analysis.
   *
   * @return string
   *   Risk level: 'low', 'medium', or 'high'.
   */
  protected function calculateSecurityRiskLevel(array $advisories, array $code_issues) {
    $risk_score = 0;
    
    // Score from security advisories
    foreach ($advisories as $advisory) {
      switch ($advisory['severity']) {
        case 'Critical':
          $risk_score += 10;
          break;
        case 'Highly Critical':
          $risk_score += 8;
          break;
        case 'Moderately Critical':
          $risk_score += 5;
          break;
        case 'Less Critical':
          $risk_score += 3;
          break;
      }
    }
    
    // Score from code issues
    $code_issue_count = 0;
    foreach ($code_issues as $file_issues) {
      $code_issue_count += count($file_issues);
    }
    
    // Add to risk score based on number of code issues
    if ($code_issue_count > 10) {
      $risk_score += 10;
    }
    elseif ($code_issue_count > 5) {
      $risk_score += 5;
    }
    elseif ($code_issue_count > 0) {
      $risk_score += 3;
    }
    
    // Determine risk level based on score
    if ($risk_score >= 15) {
      return 'high';
    }
    elseif ($risk_score >= 8) {
      return 'medium';
    }
    else {
      return 'low';
    }
  }
}
